# -*- coding: utf-8 -*-
#
#    copyright (C) 2023 Steffen Rolapp (github@rolapp.de)
#
#    based on ZattooBoxExtended by Daniel Griner (griner.ch@gmail.com) License under GPL
#    based on ZattooBox by Pascal Nançoz (nancpasc@gmail.com) Licence under BSD 2 clause
#
#    This file is part of zattooHiQ
#
#    zattooHiQ is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    zattooHiQ is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with zattooHiQ.  If not, see <http://www.gnu.org/licenses/>.
#

import sys, xbmcgui, xbmc, xbmcaddon
from resources.lib.zattooDB import ZattooDB
from resources.lib.keymap import KeyMap
from resources.lib.tools import *
from resources.lib.directorys import *
from resources.lib.vod import vod
from resources.lib.highlights import highlights
from resources.lib.guiactions import *
from urllib.parse import unquote_plus
#from resources.lib.helpmy import helpmy

#from resources.lib.myplayer import myPlayer

__addon__  = xbmcaddon.Addon()
__addonuri__    = sys.argv[0]
__addonId__     = __addon__.getAddonInfo('id')
__addondir__    = xbmcvfs.translatePath( __addon__.getAddonInfo('profile') )

_zattooDB_ = ZattooDB()
_keymap_   = KeyMap()
_vod_      = vod()
_hl_       = highlights()
#_helpmy_   = helpmy()

#accountData = _zattooDB_.zapi.exec_zapiCall('/zapi/v3/session', None)
#if __addon__.getSetting('show_favourites')=='true':_listMode_ ='favourites'
#else: _listMode_ ='all'

win=xbmcgui.Window(10000)

def showReload():
    reloadgui = reloadGUI("reload.xml",__addon__.getAddonInfo('path'))
    reloadgui.doModal()
    del reloadgui

def makeRoot():
    # load Highlights
    global hl
    hl = _hl_.hl_start()
    win.setProperty('title',hl['title'])
    # load thump
    path = os.path.join(__addondir__ + 'cache')
    if os.path.isfile(os.path.join(path, 'hl_thump.png')):
        win.setProperty('showThumb', path + '/hl_thump.png')
    else:
        win.setProperty('showThumb',hl['image_url'])
    if os.path.isfile(os.path.join(path, 'hl_logo.png')):
        win.setProperty('brand_logo', path + '/hl_logo.png')
    else:
        win.setProperty('brand_logo', hl['brand_logo'])
    #win.setProperty('List', 'False')
    win.setProperty('reccount', __addon__.getSetting('record'))
    if __addon__.getSetting('show_favourites')=='true':
        xbmc.executebuiltin( "Skin.SetBool(%s)" %'fav')
    else:
        xbmc.executebuiltin( "Skin.Reset(%s)" %'fav')
    # show help bubbles
    if __addon__.getSetting('help')=='true':
        xbmc.executebuiltin( "Skin.SetBool(%s)" %'help')
    else:
        xbmc.executebuiltin( "Skin.Reset(%s)" %'help')
    
def showRootGUI():
    makeRoot()
    #xbmc.executebuiltin("Dialog.Close(all, 'true')")
    rootgui = rootGUI("rootGUI.xml",__addon__.getAddonInfo('path'))
    rootgui.doModal()
    del rootgui

def reloadRoot():
    xbmc.executebuiltin("RunAddon(plugin.video.zattoo_hiq)")  
    
def router(params):
    #params = unquote_plus(params)
    args = urllib.parse.parse_qs(params)
    #args = dict(parse_qsl(params))
    action=args.get('mode', ['root'])[0]
    global _listMode_
    
    try:
        channel=_zattooDB_.get_playing()['channel']
        channeltitle=_zattooDB_.get_channeltitle(channel)
        program = _zattooDB_.getPrograms({'index':[channel]}, datetime.datetime.now(), datetime.datetime.now())
    except:pass
    try:
        program=program[0]
    except:pass

  # if args:
    # action = args['mode']
    xbmcgui.Window(10000).setProperty('ZBElastAction', action)
    if action == 'root': showRootGUI()
    elif action == 'channellist': build_channelsList()
    elif action == 'preview': showPreview()
    elif action == 'previewOSD': showPreview()
    elif action == 'epg': showEpg()
    elif action == 'epgOSD': showEpg()
    elif action == 'recordings': build_recordingsList()
    elif action == 'searchlist': build_searchList()
    elif action == 'selectStartChannel': selectStartChannel()
    elif action == 'editKeyMap': _keymap_.editKeyMap()
    elif action == 'deleteKeyMap': _keymap_.deleteKeyMap()
    #elif action == 'showKeyMap': showkeymap()
    elif action == 'record_l': setup_recording(program['showID'])
    elif action == 'search':
      search = args.get('id','')
      search_show(search)
    elif action == 'show_settings':
        __addon__.openSettings()
        _zattooDB_.zapi.renew_session()
    elif action == 'watch_c':
        cid = args.get('id',['ard'])
        if cid=="current": cid=_zattooDB_.get_playing()['channel']
        start = args.get('start', '0')
        end = args.get('end', '0')
        showID = args.get('showID', '1')
        restart = args.get('restart', ['false'])
        add = args.get('add', ['false'])
        watch_channel(cid, start, end, showID, restart, add)
    elif action == 'skip_channel':
        skipDir = args.get('skipDir')
        skip_channel(int(skipDir))
    #elif action == 'toggle_channel': zattooGUI.toggle_channel()
    elif action == 'switchlist':
        if __addon__.getSetting('show_favourites')=='true':
            _listMode_ ='all'
            __addon__.setSetting('show_favourites', 'false')
        else:
            _listMode_ ='favorites'
            __addon__.setSetting('show_favourites', 'true')
        __addon__.setSetting('channellist', _listMode_)
        xbmc.executebuiltin('Container.Refresh')
    elif action == 'record_p':
        program_id = args.get('program_id')
        setup_recording(program_id)
    elif action == 'watch_r':
        recording_id = args.get('id')[0]
        start = args.get('start', '0')[0]
        cid = args.get('cid')[0]
        watch_recording(recording_id, cid, start)
    elif action == 'remove_recording':
        recording_id = args.get('recording_id')
        title = args.get('title')
        delete_recording(recording_id, title)
    elif action == 'remove_series':
        recording_id = args.get('recording_id')
        series = args.get('series')
        delete_series(recording_id, series)
    elif action == 'reloadDB':
        _zattooDB_.reloadDB()
    elif action == 'changeStream':
        streams=xbmc.Player().getAvailableAudioStreams()
        dialog=xbmcgui.Dialog()
        ret = dialog.select('audio streams', streams)
        if ret==-1: return
        xbmc.Player().setAudioStream(ret)
    elif action == 'teletext':
        from .teletext import Teletext
        tele = Teletext()
        tele.doModal()
        del tele
    elif action == 'makelibrary':
        xbmc.executebuiltin("ActivateWindow(busydialog)")
        _library_.delete_library()
        _library_.make_library()
        xbmc.executebuiltin("Dialog.Close(busydialog)")
    elif action == 'resetdir':
        delete = xbmcgui.Dialog().yesno(__addonname__, __addon__.getLocalizedString(31911))
        if delete:
            _library_.delete_library()
            __addon__.setSetting(id='library_dir', value="")
            xbmc.executebuiltin('Container.Refresh')
    elif action == 'cleanProg':
        _zattooDB_.cleanProg()
    elif action == 'popular': showPreview('popular')
    elif action == 'showInfo':
        makeOsdInfo()
        osd = zattooOSD("zattooOSD.xml",__addon__.getAddonInfo('path'))
        osd.doModal()
        del osd
        
    elif action == 'nr':
        nr = args.get('nr')
        input_numbers(nr)
    
    elif action =='category':
        showCategory()
    
    elif action == 'build_category':
        cat = args.get('cat')[0]
        debug(cat)
        build_category(cat)
    elif action == 'helpmy':
        img = args.get('img')
        showHelp(img)
        
    elif action == 'showhelp': build_Help()
    
    elif action == 'inputsearch':
        search = xbmcgui.Dialog().input(__addon__.getLocalizedString(31200), type=xbmcgui.INPUT_ALPHANUM)
        if search == '': return
        search_show(search)
    
    elif action == 'deletesearch':
        dialog = xbmcgui.Dialog()
        ret = dialog.yesno(localString(320001), '[COLOR gold]'+localString(32025)+'[/COLOR]','','[COLOR red]'+local(19291)+'[/COLOR]')
        if ret == 1:
            al = args.get('al')
            search = args.get('search')
            _zattooDB_.del_search(al,search)
            xbmc.executebuiltin('Container.Refresh')
    
    elif action == 'editsearch':
        item = args.get('search')
        search = _zattooDB_.edit_search(item)
        xbmc.executebuiltin('Container.Refresh')
        
    elif action == 'vod':
        _vod_.main_menu()
    
    elif action == 'vod_sub':
        total = args.get('total')[0]
        path = args.get('path')[0]
        title = args.get('title')[0]
        _vod_.sub_menu(title, path, total)
        
    elif action == 'vod_watch':
        pfad = args.get('pfad')[0]
        params = args.get('params')[0]
        _vod_.vod_watch(pfad, params)
    
    elif action == 'add_wl':
        params = args.get('params')
        _vod_.add_watchlist(params)
    
    elif action == 'remove_wl':
        params = args.get('params')
        _vod_.remove_watchlist(params)
        
    elif action == 'clearCache':
        xbmcgui.Dialog().notification(localString(30104), localString(31024),  __addon__.getAddonInfo('path') + '/resources/icon.png', 3000, False)
        profilePath = xbmcvfs.translatePath(__addon__.getAddonInfo('profile'))
        os.remove(os.path.join(profilePath, 'cookie.cache'))
        os.remove(os.path.join(profilePath, 'session.cache'))
        os.remove(os.path.join(profilePath, 'account.cache'))
        _zattooDB_.zapiSession()
        
    elif action == 'showGUI':
        makeZattooGUI()
        
    elif action == 'logout':
        _zattooDB_.zapi.logout()
        
  # else:
    # #build_root()
    # showRootGUI() 

class rootGUI(xbmcgui.WindowXMLDialog):
    
    def __init__(self, xmlFile, scriptPath):
        super(rootGUI, self).__init__()
        xbmcgui.Window(10000).setProperty('RootGUI', 'True')
        xbmcgui.Window(10000).setProperty('List', 'False')
        self.isClosing = False
        
    def onAction(self, action):
        action = action.getId()
        #self.close()
        if action in [ACTION_PARENT_DIR, KEY_NAV_BACK, ACTION_PREVIOUS_MENU]:
            self.close()
        # if action == ACTION_SKIPPREW:
            # xbmc.executebuiltin("Action(Left)")
        # elif action == ACTION_SKIPNEXT:
            # xbmc.executebuiltin("Action(Right)")
    def onClick(self, controlID):

        if controlID==200: #preview
            showPreview()
            xbmc.executebuiltin("RunAddon(plugin.video.zattoo_hiq)")
        elif controlID==201: #popular
            showPreview('popular')
            xbmc.executebuiltin("RunAddon(plugin.video.zattoo_hiq)")
        elif controlID==202: #epg
            showEpg()
            xbmc.executebuiltin("Dialog.Close(all, true)")
        elif controlID==203: # Kanalliste
            self.close()
            xbmcgui.Window(10000).setProperty('zattoo_runningView','list')
            build_channelsList()
            while xbmcgui.getCurrentWindowId() == 10025:
                xbmc.sleep(200)
            reloadRoot()
        elif controlID==204: # kat
            self.close()
            showCategory()
            while xbmcgui.getCurrentWindowId() == 10025:
                xbmc.sleep(200)
            reloadRoot()
        elif controlID==205: # Search
            self.close()
            build_searchList()
            while xbmcgui.getCurrentWindowId() == 10025:
                xbmc.sleep(200)
            reloadRoot()
        elif controlID==206: #vod
            self.close()
            _vod_.main_menu()
            while xbmcgui.getCurrentWindowId() == 10025:
                xbmc.sleep(200)
            reloadRoot()
        elif controlID==207: #recordlist
            self.close()
            build_recordingsList()
            reloadRoot()
        elif controlID==300: #favourites
            if __addon__.getSetting('show_favourites')=='true':
                _listMode_ ='all'
                __addon__.setSetting('show_favourites', 'false')
                xbmc.executebuiltin( "Skin.Reset(%s)" %'fav')
                xbmcgui.Dialog().notification(local(1036), local(1223), xbmcgui.NOTIFICATION_INFO, 1500, False)
            else:
                _listMode_ ='favorites'
                __addon__.setSetting('show_favourites', 'true')
                xbmc.executebuiltin( "Skin.SetBool(%s)" %'fav')
                xbmcgui.Dialog().notification(local(1036), local(305), xbmcgui.NOTIFICATION_INFO, 1500, False)
            __addon__.setSetting('channellist', _listMode_)

        elif controlID==301: #Addon Settings
            self.close()
            __addon__.openSettings()  
        elif controlID==302: #Addon Help
            self.close()
            build_Help()
            reloadRoot()
        elif controlID==303: #show Reload menu
            showReload()
        
        elif controlID==304: #back to Video addons
            self.close()
            xbmc.excutebuiltin("ActivateWindow(Videos,addons://sources/video/)")
        elif controlID==305: #window_home
            self.close()
            xbmc.executebuiltin("Dialog.Close(all, 'true')")
            win=xbmcgui.Window(10000)
            win.setProperty('List', 'False')
            xbmc.executebuiltin("ActivateWindow(Home)")  
            
        elif controlID==100: # play Highlight
            self.close()
            _hl_.hl_stream(hl)

    def onFocus(self, controlID):
        i=10    
        
    def close(self):
        xbmcgui.Window(10000).setProperty('zattooRoot', 'False')
        super(rootGUI, self).close()
        if not self.isClosing:
            self.isClosing = True
            super(rootGUI, self).close()

class SettingMonitor(xbmc.Monitor):
    def __init__(self, *args, **kwargs):
        xbmc.Monitor.__init__(self)
        
    def onSettingsChanged(self):
        #debug('Setting change')
        xbmc.sleep(200)
        #debug(__addon__.getSetting('channellist'))
        if __addon__.getSetting('show_favourites') == 'true':
            xbmcgui.Window(10000).setProperty('favourites', 'true')
        else:
            xbmcgui.Window(10000).setProperty('favourites', 'false')
            
class reloadGUI(xbmcgui.WindowXMLDialog):
    
    def __init__(self, xmlFile, scriptPath):
        super(reloadGUI, self).__init__()
        self.isClosing = False
    
    def onClick(self, controlID):
        if controlID==1000: #reload DatabaseError
            _zattooDB_.reloadDB()
            self.close()
            #xbmc.executebuiltin("RunAddon(plugin.video.zattoo_hiq)")  
            
        if controlID==1001: #reload DatabaseError
            _zattooDB_.zapi.logout()
            _zattooDB_.zapi.renew_session()
            self.close()
            
    def close(self):
        super(reloadGUI, self).close()
        if not self.isClosing:
            self.isClosing = True
            super(reloadGUI, self).close()

if __name__ == '__main__':   
    debug('script-start')
    monsettings = SettingMonitor()
    router(sys.argv[2][1:])
    #router(sys.argv[2])

