# # -*- coding: utf-8 -*-
#
#    copyright (C) 2023 Steffen Rolapp (github@rolapp.de)
#
#    based on ZattooBoxExtended by Daniel Griner (griner.ch@gmail.com) license under GPL
#
#    This file is part of ZattooHiQ
#
#    zattooHiQ is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    zattooHiQ is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with zattooHiQ.  If not, see <http://www.gnu.org/licenses/>.
#

import xbmc, xbmcgui, xbmcplugin, xbmcaddon, xbmcvfs
import sys, urllib.parse, urllib.request, urllib.error
from .zattooDB import ZattooDB
from .vod import vod
from .myplayer import myPlayer

_zattooDB_ = ZattooDB()
_vod_ = vod()

__addon__       = xbmcaddon.Addon()
__addonId__     =__addon__.getAddonInfo('id')
__addonname__   = __addon__.getAddonInfo('name')
__addondir__    = xbmcvfs.translatePath( __addon__.getAddonInfo('profile') )
__addonhandle__ = int(sys.argv[1])

DEBUG           = __addon__.getSetting('debug')
STREAM_TYPE     = 'dash'
COUNTRY         = __addon__.getSetting('contry')

def debug(content):
    if DEBUG:log(content, xbmc.LOGDEBUG)
    
def notice(content):
    log(content, xbmc.LOGINFO)

def log(msg, level=xbmc.LOGINFO):
    addon = xbmcaddon.Addon()
    addonID = addon.getAddonInfo('id')
    xbmc.log('%s: %s' % (addonID, msg), level) 

class highlights:
    def __init__(self):
        self.zapi = _zattooDB_.zapiSession()
        
    def hl_start(self):
        if COUNTRY == 'CH': country = 'highlights_tonight_ch_de'
        else: country = 'highlights_tonight_de'
        api = '/zapi/v2/cached/' + self.zapi.AccountData['power_guide_hash'] + '/teaser_collections/' + country
        hl_main = self.zapi.exec_zapiCall(api, None)
        
        for teasers in hl_main['teasers']:
            debug(teasers)
            meta = {
                't_type': teasers['teasable_type'],
                't_id':   teasers['teasable_id'],
                'title':  teasers['title'],
                'data':   teasers['teasable'],
                'image_url':  'https://images.zattic.com/cms/' + teasers['image_token'] + '/format_480x360.jpg',
                }
            debug(teasers['teasable_type'] )
            if teasers['teasable']['brand']['logo_token']: 
                meta['brand_logo'] = 'https://images.zattic.com/logos/' + teasers['teasable']['brand']['logo_token'] + '/black/42x24.png'
            break    

        return meta
    
    def hl_stream(self, meta):
        debug(meta)
        t_type = meta['t_type']
        t_id = meta['t_id']
        data = meta['data']
        
        #if t_type == "Vod::Episode":
        pfad = 'watch/vod/video'
        token = data['terms_catalog'][0]['terms'][0]['token']
        debug(token)
        params = {
                'term_token': str(token),
                'teasable_id': str(t_id),
                'teasable_type': t_type,
                'stream_type': 'dash',
                #'youth_protection_pin': YPIN,
                'max_drm_lvl':'3'
                }
        
        resultData = self.zapi.exec_zapiCall('/zapi/' + pfad , params)
        debug ('ResultData: '+str(resultData))
        if resultData is not None:
            streams = resultData['stream']['watch_urls']
            
            if len(streams)==0:
                xbmcgui.Dialog().notification("ERROR", "NO STREAM FOUND, CHECK SETTINGS!", ICON_PATH, 5000, False)
                return
            elif len(streams) > 1 and  __addon__.getSetting('audio_stream') == 'B' and streams[1]['audio_channel'] == 'B': streamNr = 1
            else: streamNr = 0
        debug(streams[0]['url'])
        li = xbmcgui.ListItem(path=streams[streamNr]['url'])
        if STREAM_TYPE == 'dash':
            li.setProperty('inputstream', 'inputstream.adaptive')
            li.setProperty('inputstream.adaptive.manifest_type', 'mpd')
        elif STREAM_TYPE == 'dash_widevine':
            li.setProperty('inputstream', 'inputstream.adaptive')
            li.setProperty('inputstream.adaptive.manifest_type', 'mpd')
            li.setProperty('inputstream.adaptive.license_key', streams[streamNr]['license_url'] + "||a{SSM}|")
            li.setProperty('inputstream.adaptive.license_type', "com.widevine.alpha")
            
        li.setProperty('isplayable', 'true')
        li.setIsFolder(False)
            
        #xbmcplugin.setResolvedUrl(__addonhandle__, True, li)
        player = xbmc.Player()
        player.play(streams[0]['url'], li)
        while xbmcgui.getCurrentWindowId() == 12005: xbmc.sleep(200)
        xbmc.executebuiltin("Dialog.Close(all, 'true')")
        xbmc.executebuiltin("RunAddon(plugin.video.zattoo_hiq)")  

                    
            
                     
        
    
